;
;	>>> ISO-Chunky Plasma and Rotator by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;
;	10x26 pixel, full color plasma and rotator.
;
;	Note: Rotator rdegtab and lumtab cross a page boundary $104 bytes.
;	Note: Plasma needs an overrun area in the sinus256.
;
;	This module requires 3 volatile bytes.
;
;	@com.wudsn.ide.asm.hardware=ATARI2600
;	@com.wudsn.ide.asm.mainsourcefile=ISO.asm

	.proc chunky
	
rows		= 26
row_height	= 2
row_cnt		= volatile_zp	;1 byte
row_temp	= volatile_zp+1	;1 byte
pattern		= volatile_zp+2	;1 byte

animate_row_ptr = chunky_zp	;2 bytes
chunky_cnt	= chunky_zp+2
chunky_add	= chunky_zp+3
chunky_zp_ram	= chunky_zp+4
chunky_zp_end	= chunky_zp_ram + .len chunky_zp_template


.if chunky_zp_end > zp_end
	.error "ZP chunky_zp:	", chunky_zp, " - ", chunky_zp_end-1, " (", chunky_zp_end-chunky_zp, " bytes)"
.else
	.print "ZP chunky_zp:	", chunky_zp, " - ", chunky_zp_end-1, " (", chunky_zp_end-chunky_zp, " bytes)"
.endif

;===============================================================

;	IN: <A>=<#animate_row, <X>=>#animate_row
	.proc init
	sta animate_row_ptr
	stx animate_row_ptr+1

	ldx #.len chunky_zp_template - 1
copy_rom
	lda chunky_zp_rom,x		;Do not use chunky_zp_template as it points to the ZP
	sta chunky_zp_ram,x
	dex
	bpl copy_rom
	rts
	.endp

;	IN: 
	.proc playfield
	jsr wsync2
	lda #14
	sta wsync
	sta colubk
	lda #0
	sta wsync
	sta colubk
	tax
	tay
	sta row_temp
	sta ctrlpf
	lda #rows
	sta row_cnt
	lda #$33
	sta pattern
row_loop
	jmp (animate_row_ptr)	;Start animation routine with row_temp:=X:=Y:=0
row_start_plus_10
	txa
	clc
	adc #10
	sta row_temp
row_start
	ldx #row_height
	lda pattern
	sta pf1
	sta wsync		;3 cycles + x sync cycles
	bmi row_delay3cyles	;delay 2/3 cycles
	sta $ffff		;delay 2+4 cycles
row_delay3cyles
	eor #$ff		;2 cycles
	sta pf2			;3 cycles
	sta pf0			;3 cycles
	sta pattern		;3 cycles
	jmp chunky_zp_template.kernel	;3 cycles
row_return
	dec row_cnt
	bne row_loop
wsync2	sta wsync
	sta wsync
	rts
	.endp

;===============================================================

	.proc plasma1
	
	.proc init
one	lda #0
	.byte $2c
two	lda #1
	sta chunky_add
	
	m_set_effect vblank playfield
	lda #<animate_row
	ldx #>animate_row
	jmp chunky.init
	.endp

;===============================================================

	.proc vblank
	inc chunky_cnt
	rts
	.endp		;End of vblank

;===============================================================

	.proc playfield
	jmp chunky.playfield
	.endp		;End of playfield

	.macro m_animate
	ldx chunky_cnt
	lda sinus256+2*:1,x
	adc sinus256+2*:1,y
	lsr
	tax
	lda coltab,x				;4 cycles
	sta chunky_zp_template.kernel.color:1	;4 cycles
	.endm

	.proc animate_row
	sec
	lda chunky_add
	adc row_temp
	sta row_temp
	tay

	m_animate 0
	m_animate 1
	m_animate 2
	m_animate 3
	m_animate 4
	m_animate 5
	m_animate 6
	m_animate 7
	m_animate 8
	m_animate 9
	jmp chunky.playfield.row_start
	.endp		;End of animate_row

	.endp		;End of plasma

	.print "PROC plasma1:	", plasma1, " - ", plasma1+.len plasma1-1, " (", .len plasma1, " bytes)"

;===============================================================

	.proc plasma2
	
	.proc init
	m_set_effect vblank playfield
	lda #<animate_row
	ldx #>animate_row
	jmp chunky.init
	.endp

;===============================================================

	.proc vblank
	inc chunky_cnt
	rts
	.endp		;End of vblank

;===============================================================

	.proc playfield
	jmp chunky.playfield
	.endp		;End of playfield

	.macro m_animate
	ldx chunky_cnt
	lda sinus256+3*:1,x
	adc sinus256+3*:1,y
	lsr
	tax
	lda coltab,x				;4 cycles
	sta chunky_zp_template.kernel.color:1	;4 cycles
	.endm

	.proc animate_row
	inc row_temp
	ldy row_temp

	m_animate 0
	m_animate 1
	m_animate 2
	m_animate 3
	m_animate 4
	m_animate 5
	m_animate 6
	m_animate 7
	m_animate 8
	m_animate 9
	jmp chunky.playfield.row_start
	.endp		;End of animate_row

	.endp		;End of plasma

	.print "PROC plasma2:	", plasma2, " - ", plasma2+.len plasma2-1, " (", .len plasma2, " bytes)"

;===============================================================

	.proc rotator

	.proc init
	m_set_effect vblank playfield

wheel	lda #<animate_row_wheel
	ldx #>animate_row_wheel
store	jmp chunky.init

circle	lda #<animate_row_circle
	ldx #>animate_row_circle
	bne store

twirl	lda #<animate_row_twirl
	ldx #>animate_row_twirl
	bne store
	.endp

;===============================================================

	.proc vblank
	dec chunky_cnt
	ldy chunky_cnt
	lda sinus256,y
	lsr
	sta chunky_add
	rts
	.endp			;End of vblank

;===============================================================

	.proc playfield
	jmp chunky.playfield
	.endp			;End of playfield

;===============================================================

	.proc animate_row_wheel
	
	.macro m_animate
;	clc
	lda degtab+:1,x
	asl
	adc chunky_add
	tay
	lda coltab,y			;4
	sta chunky_zp_template.kernel.color:1	;3
	.endm

	ldx row_temp
	m_animate 0
	m_animate 1
	m_animate 2
	m_animate 3
	m_animate 4
	m_animate 5
	m_animate 6
	m_animate 7
	m_animate 8
	m_animate 9
	jmp playfield.row_start_plus_10

	.endp			;End of animate_row_wheel

	.print "PROC rotator.animate_row_wheel:	", animate_row_wheel, " - ", animate_row_wheel+.len animate_row_wheel-1, " (", .len animate_row_wheel, " bytes)"

;===============================================================

	.proc animate_row_circle
	
	.macro m_animate
;	clc
	lda chunky_add
	adc lumtab+:1,x
	tay
	bit $80
	bpl skip
	lda coltab,y			;4
skip	sta chunky_zp_template.kernel.color:1	;3
	.endm

	ldx row_temp
	m_animate 0
	m_animate 1
	m_animate 2
	m_animate 3
	m_animate 4
	m_animate 5
	m_animate 6
	m_animate 7
	m_animate 8
	m_animate 9
	jmp playfield.row_start_plus_10

	.endp			;End of animate_row_circle

	.print "PROC rotator.animate_row_circle:	", animate_row_circle, " - ", animate_row_circle+.len animate_row_circle-1, " (", .len animate_row_circle, " bytes)"

;===============================================================

	.proc animate_row_twirl
	
	.macro m_animate
	lda degtab+:1,x
	asl
	adc chunky_add
	adc lumtab+:1,x
	tay
	lda coltab,y			;4
	sta chunky_zp_template.kernel.color:1	;3
	.endm

	ldx row_temp
	m_animate 0
	m_animate 1
	m_animate 2
	m_animate 3
	m_animate 4
	m_animate 5
	m_animate 6
	m_animate 7
	m_animate 8
	m_animate 9
	jmp playfield.row_start_plus_10

	.endp			;End of animate_row_twirl

	.print "PROC rotator.animate_row_twirl:	", animate_row_twirl, " - ", animate_row_twirl+.len animate_row_twirl-1, " (", .len animate_row_twirl, " bytes)"

;===============================================================

	.endp			;End of rotator

	.print "PROC rotator:	", rotator, " - ", rotator+.len rotator-1, " (", .len rotator, " bytes)"

;===============================================================
;	In: <X>=8 (number of lines)
;	Out: <X>=0; Z=1

chunky_zp_rom

	.proc chunky_zp_template, chunky_zp_ram

	.proc kernel
	colreg = colupf

loop	lda #0			;10*5+1 = 51 cycles
color0 = *-1
	sta colreg
	lda #0
color1 = *-1
	sta colreg
	lda #0
color2 = *-1
	sta colreg
	lda #0
color3 = *-1
	sta colreg
	lda #0
color4 = *-1
	sta colreg
	lda #0
color5 = *-1
	sta colreg
	lda #0
color6 = *-1
	sta colreg+$100	;+1 cycle
	lda #0
color7 = *-1
	sta colreg
	lda #0
color8 = *-1
	sta colreg
	lda #0
color9 = *-1
	sta colreg

	jsr wait.wait20
	dex		;2 cycles
	bne loop	;3 cycles normally, 2 on exit
	stx colreg	;Black playfield
	jmp playfield.row_return

	.endp		;End of chunky_zp_kernel
	.endp		;End of chunky_zp_template

	.print "PROC chunky_zp_template:	", chunky_zp_template, " - ", chunky_zp_template+.len chunky_zp_template-1, " (", .len chunky_zp_template, " bytes)"
	
;===============================================================

	.endp		;End of chunky
	.print "PROC chunky:	", chunky, " - ", chunky+.len chunky-1, " (", .len chunky, " bytes)"
